@@ -101,7 +101,7 @@ void xalloc_die(void)
 
 char *mpop_password_callback(const char *hostname, const char *user)
 {
-    return password_get(hostname, user, password_service_pop3, 0);
+    return password_get(hostname, user, password_service_pop3, 1, 0);
 }
 
 
@@ -2274,12 +2274,12 @@ int main(int argc, char *argv[])
             free(conffile);
         }
         printf("\n");
-        printf(_("Copyright (C) 2021 Martin Lambers and others.\n"
+        printf(_("Copyright (C) %d Martin Lambers and others.\n"
                     "This is free software.  You may redistribute copies of "
                         "it under the terms of\n"
                     "the GNU General Public License "
                         "<http://www.gnu.org/licenses/gpl.html>.\n"
-                    "There is NO WARRANTY, to the extent permitted by law.\n"));
+                    "There is NO WARRANTY, to the extent permitted by law.\n"), 2021);
     }
     if (print_help)
     {
@@ -2433,8 +2433,8 @@ int main(int argc, char *argv[])
         account = lp->data;
         if (!account->password && account->passwordeval)
         {
-            if (get_password_eval(account->passwordeval,
-                        &account->password, &errstr) != CONF_EOK)
+            if (password_eval(account->passwordeval,
+                        &account->password, &errstr) != 0)
             {
                 print_error("%s", sanitize_string(errstr));
                 error_code = EX_CONFIG;
