@@ -1,9 +1,10 @@
 /*
  * net.c
  *
- * This file is part of msmtp, an SMTP client.
+ * This file is part of msmtp, an SMTP client, and of mpop, a POP3 client.
  *
- * Copyright (C) 2000, 2003, 2004, 2005, 2006, 2007, 2008, 2012, 2014, 2015
+ * Copyright (C) 2000, 2003, 2004, 2005, 2006, 2007, 2008, 2012, 2014, 2015,
+ * 2018, 2019, 2020
  * Martin Lambers <marlam@marlam.de>
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -25,17 +26,29 @@
 # include "config.h"
 #endif
 
+#ifdef W32_NATIVE
+# define WIN32_LEAN_AND_MEAN    /* do not include more than necessary */
+# define _WIN32_WINNT 0x0601    /* Windows 7 or later */
+# include <winsock2.h>
+# include <ws2tcpip.h>
+#endif
 #include <stdlib.h>
-#include <stdio.h>
 #include <stdint.h>
 #include <string.h>
+#include <limits.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <errno.h>
 #include <sys/time.h>
-#include <sys/types.h>
+#ifdef HAVE_SYS_SELECT_H
+# include <sys/select.h>
+#endif
 #ifdef HAVE_SYS_SOCKET_H
 # include <sys/socket.h>
+# include <sys/un.h>
+#endif
+#ifdef HAVE_NETINET_IN_H
+# include <netinet/in.h>
 #endif
 #ifdef HAVE_ARPA_INET_H
 # include <arpa/inet.h>
@@ -43,15 +56,14 @@
 #ifdef HAVE_NETDB_H
 # include <netdb.h>
 #endif
-#ifdef W32_NATIVE
-# define WIN32_LEAN_AND_MEAN    /* do not include more than necessary */
-# define _WIN32_WINNT 0x0502    /* Windows XP SP2 or later */
-# include <winsock2.h>
-# include <ws2tcpip.h>
-#endif
 
 #ifdef HAVE_LIBIDN
-# include <idna.h>
+# include <idn2.h>
+#endif
+
+#ifdef HAVE_LIBRESOLV
+# include <arpa/nameser.h>
+# include <resolv.h>
 #endif
 
 #include "gettext.h"
@@ -59,6 +71,7 @@
 
 #include "xalloc.h"
 #include "readbuf.h"
+#include "tools.h"
 #include "net.h"
 
 
@@ -179,7 +192,7 @@ int net_recv(int fd, void *buf, size_t len, char **errstr)
     {
 #ifdef W32_NATIVE
         int e = WSAGetLastError();
-        if (e == WSAEWOULDBLOCK)
+        if (e == WSAETIMEDOUT)
         {
             *errstr = xasprintf(_("network read error: %s"),
                     _("the operation timed out"));
@@ -226,7 +239,7 @@ int net_send(int fd, const void *buf, size_t len, char **errstr)
     {
 #ifdef W32_NATIVE
         int e = WSAGetLastError();
-        if (e == WSAEWOULDBLOCK)
+        if (e == WSAETIMEDOUT)
         {
             *errstr = xasprintf(_("network write error: %s"),
                     _("the operation timed out"));
@@ -305,6 +318,45 @@ void net_close_socket(int fd)
 }
 
 
+/*
+ * net_bind_source_ip_to_socket()
+ *
+ * This function binds a source IP (in string representation, either IPv6 or IPv4)
+ * to a socket. It behaves like bind() in terms of return value and errno.
+ */
+
+int net_bind_source_ip_to_socket(int fd, const char *source_ip)
+{
+    struct sockaddr_in6 sa6;
+    struct sockaddr_in sa4;
+
+    memset(&sa6, 0, sizeof(sa6));
+    if (inet_pton(AF_INET6, source_ip, &sa6.sin6_addr) != 0)
+    {
+        sa6.sin6_family = AF_INET6;
+        return bind(fd, (struct sockaddr *)&sa6, sizeof(sa6));
+    }
+    else
+    {
+        memset(&sa4, 0, sizeof(sa4));
+        if (inet_pton(AF_INET, source_ip, &sa4.sin_addr) != 0)
+        {
+            sa4.sin_family = AF_INET;
+            return bind(fd, (struct sockaddr *)&sa4, sizeof(sa4));
+        }
+        else
+        {
+#ifdef W32_NATIVE
+            WSASetLastError(WSAEINVAL);
+#else
+            errno = EINVAL;
+#endif
+            return -1;
+        }
+    }
+}
+
+
 /*
  * net_connect()
  *
@@ -319,16 +371,17 @@ int net_connect(int fd, const struct sockaddr *serv_addr, socklen_t addrlen,
         int timeout)
 {
 #ifdef W32_NATIVE
-    /* TODO: I don't know how to do this on Win32. Please send a patch. */
-    return connect(fd, serv_addr, addrlen);
-#else /* UNIX or DJGPP */
-
+    u_long flags;
+    DWORD err;
+    int optlen;
+    fd_set eset;
+#else
     int flags;
-    struct timeval tv;
-    fd_set rset;
-    fd_set wset;
     int err;
     socklen_t optlen;
+#endif
+    struct timeval tv;
+    fd_set wset;
 
     if (timeout <= 0)
     {
@@ -337,8 +390,13 @@ int net_connect(int fd, const struct sockaddr *serv_addr, socklen_t addrlen,
     else
     {
         /* make socket non-blocking */
+#ifdef W32_NATIVE
+        flags = 1;
+        if (ioctlsocket(fd, FIONBIO, &flags) == SOCKET_ERROR)
+#else
         flags = fcntl(fd, F_GETFL, 0);
         if (fcntl(fd, F_SETFL, flags | O_NONBLOCK) == -1)
+#endif
         {
             return -1;
         }
@@ -346,51 +404,75 @@ int net_connect(int fd, const struct sockaddr *serv_addr, socklen_t addrlen,
         /* start connect */
         if (connect(fd, serv_addr, addrlen) < 0)
         {
+#ifdef W32_NATIVE
+            if (WSAGetLastError() != WSAEWOULDBLOCK)
+#else
             if (errno != EINPROGRESS)
+#endif
             {
                 return -1;
             }
 
             tv.tv_sec = timeout;
             tv.tv_usec = 0;
-            FD_ZERO(&rset);
             FD_ZERO(&wset);
-            FD_SET(fd, &rset);
             FD_SET(fd, &wset);
+#ifdef W32_NATIVE
+            FD_ZERO(&eset);
+            FD_SET(fd, &eset);
+#endif
 
             /* wait for connect() to finish */
-            if ((err = select(fd + 1, &rset, &wset, NULL, &tv)) <= 0)
+#ifdef W32_NATIVE
+            /* In case of an error on connect(), eset will be affected instead
+             * of wset (on Windows only). */
+            if ((err = select(fd + 1, NULL, &wset, &eset, &tv)) <= 0)
+#else
+            if ((err = select(fd + 1, NULL, &wset, NULL, &tv)) <= 0)
+#endif
             {
                 /* errno is already set if err < 0 */
                 if (err == 0)
                 {
+#ifdef W32_NATIVE
+                    WSASetLastError(WSAETIMEDOUT);
+#else
                     errno = ETIMEDOUT;
+#endif
                 }
                 return -1;
             }
 
             /* test for success, set errno */
-            optlen = sizeof(int);
+            optlen = sizeof(err);
             if (getsockopt(fd, SOL_SOCKET, SO_ERROR, &err, &optlen) < 0)
             {
                 return -1;
             }
             if (err != 0)
             {
+#ifdef W32_NATIVE
+                WSASetLastError(err);
+#else
                 errno = err;
+#endif
                 return -1;
             }
         }
 
         /* restore blocking mode */
+#ifdef W32_NATIVE
+        flags = 0;
+        if (ioctlsocket(fd, FIONBIO, &flags) == SOCKET_ERROR)
+#else
         if (fcntl(fd, F_SETFL, flags) == -1)
+#endif
         {
             return -1;
         }
 
         return 0;
     }
-#endif /* UNIX */
 }
 
 
@@ -411,7 +493,7 @@ void net_set_io_timeout(int socket, int seconds)
         (void)setsockopt(socket, SOL_SOCKET, SO_RCVTIMEO, &milliseconds, sizeof(int));
         (void)setsockopt(socket, SOL_SOCKET, SO_SNDTIMEO, &milliseconds, sizeof(int));
     }
-#else /* UNIX or DJGPP */
+#else /* UNIX */
     struct timeval tv;
 
     if (seconds > 0)
@@ -434,8 +516,7 @@ void net_set_io_timeout(int socket, int seconds)
 int net_socks5_connect(int fd, const char *hostname, int port, char **errstr)
 {
     /* maximum size of a SOCKS5 message (send or receive) */
-    const size_t required_buffer_size = 1 + 1 + 1 + 1 + 1 + 255 + 2;
-    unsigned char buffer[required_buffer_size];
+    unsigned char buffer[1 + 1 + 1 + 1 + 1 + 255 + 2];
     size_t hostname_len = strlen(hostname);
     uint16_t nport = htons(port);
     size_t len;
@@ -570,8 +651,10 @@ int net_socks5_connect(int fd, const char *hostname, int port, char **errstr)
 }
 
 int net_open_socket(
+        const char *socketname,
         const char *proxy_hostname, int proxy_port,
         const char *hostname, int port,
+        const char *source_ip,
         int timeout,
         int *ret_fd, char **canonical_name, char **address,
         char **errstr)
@@ -587,10 +670,51 @@ int net_open_socket(
     char nameinfo_buffer[NI_MAXHOST];
     char *idn_hostname = NULL;
 
+    if (socketname)
+    {
+#ifdef W32_NATIVE
+        *errstr = xasprintf(_("cannot connect to %s: %s"), socketname,
+                wsa_strerror(WSAESOCKTNOSUPPORT));
+        return NET_ELIBFAILED;
+#else
+        struct sockaddr_un addr;
+        if (strlen(socketname) + 1 > sizeof(addr.sun_path))
+        {
+            *errstr = xasprintf(_("cannot connect to %s: %s"), socketname,
+                    _("invalid argument"));
+            return NET_EIO;
+        }
+        if ((fd = socket(AF_UNIX, SOCK_STREAM, 0)) == -1)
+        {
+            *errstr = xasprintf(_("cannot create socket: %s"), strerror(errno));
+            return NET_ESOCKET;
+        }
+        memset(&addr, 0, sizeof(addr));
+        addr.sun_family = AF_UNIX;
+        strcpy(addr.sun_path, socketname);
+        if (net_connect(fd, (struct sockaddr *)&addr, sizeof(addr), timeout) < 0)
+        {
+            net_close_socket(fd);
+            *errstr = xasprintf(_("cannot connect to %s: %s"), socketname, strerror(errno));
+            return NET_ECONNECT;
+        }
+        *ret_fd = fd;
+        if (canonical_name)
+        {
+            *canonical_name = NULL;
+        }
+        if (address)
+        {
+            *address = NULL;
+        }
+        return NET_EOK;
+#endif
+    }
+
     if (proxy_hostname)
     {
-        error_code = net_open_socket(NULL, -1, proxy_hostname, proxy_port,
-                timeout, &fd, NULL, NULL, errstr);
+        error_code = net_open_socket(NULL, NULL, -1, proxy_hostname, proxy_port,
+                source_ip, timeout, &fd, NULL, NULL, errstr);
         if (error_code != NET_EOK)
         {
             return error_code;
@@ -626,7 +750,7 @@ int net_open_socket(
     hints.ai_flags |= AI_IDN;
 # endif
 #elif defined(HAVE_LIBIDN)
-    idna_to_ascii_lz(hostname, &idn_hostname, 0);
+    idn2_to_ascii_lz(hostname, &idn_hostname, IDN2_NFC_INPUT | IDN2_NONTRANSITIONAL);
 #endif
     error_code = getaddrinfo(idn_hostname ? idn_hostname : hostname,
             port_string, &hints, &res0);
@@ -668,9 +792,21 @@ int net_open_socket(
 #endif
             continue;
         }
-        if (net_connect(fd, res->ai_addr, res->ai_addrlen, timeout) < 0)
+        if (source_ip && net_bind_source_ip_to_socket(fd, source_ip) != 0)
         {
             cause = 2;
+#ifdef W32_NATIVE
+            failure_errno = WSAGetLastError();
+#else
+            failure_errno = errno;
+#endif
+            net_close_socket(fd);
+            fd = -1;
+            continue;
+        }
+        if (net_connect(fd, res->ai_addr, res->ai_addrlen, timeout) < 0)
+        {
+            cause = 3;
 #ifdef W32_NATIVE
             if (WSAGetLastError() != WSAENETUNREACH)
             {
@@ -682,7 +818,7 @@ int net_open_socket(
                 failure_errno = errno;
             }
 #endif
-            close(fd);
+            net_close_socket(fd);
             fd = -1;
             continue;
         }
@@ -732,7 +868,18 @@ int net_open_socket(
                     );
             return NET_ESOCKET;
         }
-        else /* cause == 2 */
+        else if (cause == 2)
+        {
+            *errstr = xasprintf(_("cannot bind source ip %s: %s"), source_ip,
+#ifdef W32_NATIVE
+                    wsa_strerror(failure_errno)
+#else
+                    strerror(failure_errno)
+#endif
+                    );
+            return NET_ESOCKET;
+        }
+        else /* cause == 3 */
         {
 #ifdef W32_NATIVE
             if (failure_errno == 0)
@@ -869,17 +1016,24 @@ int net_puts(int fd, const char *s, size_t len, char **errstr)
  * see net.h
  */
 
-char *net_get_canonical_hostname(void)
+char *net_get_canonical_hostname(const char *hostname)
 {
-    char hostname[256];
+    char buf[256];
     char *canonname = NULL;
     struct addrinfo hints;
     struct addrinfo *res0;
 
-    if (gethostname(hostname, 256) == 0)
+    if (!hostname)
+    {
+        if (gethostname(buf, 256) == 0)
+        {
+            /* Make sure the hostname is NUL-terminated. */
+            buf[255] = '\0';
+            hostname = buf;
+        }
+    }
+    if (hostname)
     {
-        /* Make sure the hostname is NUL-terminated. */
-        hostname[255] = '\0';
         hints.ai_family = PF_UNSPEC;
         hints.ai_socktype = 0;
         hints.ai_flags = AI_CANONNAME;
@@ -897,7 +1051,10 @@ char *net_get_canonical_hostname(void)
             freeaddrinfo(res0);
         }
     }
-
+    if (!canonname && hostname)
+    {
+        canonname = xstrdup(hostname);
+    }
     if (!canonname)
     {
         canonname = xstrdup("localhost");
@@ -907,6 +1064,89 @@ char *net_get_canonical_hostname(void)
 }
 
 
+/*
+ * net_get_srv_query()
+ *
+ * see net.h
+ */
+char* net_get_srv_query(const char *domain, const char *service)
+{
+    size_t domain_len = strlen(domain);
+    size_t service_len = strlen(service);
+    size_t query_len = 1 /* '_' */ + service_len + 6 /* "._tcp." */ + domain_len;
+    char* query = xmalloc(query_len + 1);
+    query[0] = '_';
+    strncpy(query + 1, service, service_len);
+    strncpy(query + 1 + service_len, "._tcp.", 6);
+    strcpy(query + 1 + service_len + 6, domain);
+    return query;
+}
+
+
+/*
+ * net_get_srv()
+ *
+ * see net.h
+ */
+int net_get_srv_record(const char* query, char **hostname, int *port)
+{
+#ifdef HAVE_LIBRESOLV
+
+    unsigned char buffer[NS_PACKETSZ];
+    int response_len;
+    ns_msg msg;
+    int i;
+    int current_prio = INT_MAX;
+    int current_weight = -1;
+    char *current_hostname = NULL;
+    int current_port = 0;
+
+    response_len = res_query(query, ns_c_in, ns_t_srv, buffer, sizeof(buffer));
+    if (response_len < 0) {
+        return NET_ESRVNOTFOUND;
+    }
+
+    ns_initparse(buffer, response_len, &msg);
+
+    for (i = 0; i < ns_msg_count(msg, ns_s_an); i++) {
+        ns_rr rr;
+        if (ns_parserr(&msg, ns_s_an, i, &rr))
+            continue; /* don't know what's wrong; ignore this part */
+        if (ns_rr_type(rr) == ns_t_srv) {
+            char name[NI_MAXHOST];
+            int prio, weight;
+            if (dn_expand(ns_msg_base(msg), ns_msg_end(msg), ns_rr_rdata(rr) + 6, name, sizeof(name)) < 0)
+                continue; /* don't know what's wrong; ignore this part */
+            if (name[0] == '\0')
+                continue; /* empty host name; ignore this part */
+            prio = ntohs(*((unsigned short*)ns_rr_rdata(rr) + 0));
+            weight = ntohs(*((unsigned short*)ns_rr_rdata(rr) + 1));
+            if (prio < current_prio || (prio == current_prio && weight > current_weight)) {
+                free(current_hostname);
+                current_hostname = xstrdup(name);
+                current_port = ntohs(*((unsigned short*)ns_rr_rdata(rr) + 2));
+                current_prio = prio;
+                current_weight = weight;
+            }
+        }
+    }
+    if (!current_hostname) {
+        /* the loop finished but we did not find usable information */
+        return NET_EIO;
+    } else {
+        *hostname = current_hostname;
+        *port = current_port;
+        return NET_EOK;
+    }
+
+#else
+
+    return NET_ELIBFAILED;
+
+#endif
+}
+
+
 /*
  * net_lib_deinit()
  *
@@ -919,3 +1159,30 @@ void net_lib_deinit(void)
     (void)WSACleanup();
 #endif
 }
+
+
+/*
+ * net_exitcode()
+ *
+ * see net.h
+ */
+
+int net_exitcode(int net_error_code)
+{
+    switch (net_error_code)
+    {
+        case NET_EHOSTNOTFOUND:
+            return EX_NOHOST;
+        case NET_ESOCKET:
+            return EX_OSERR;
+        case NET_ECONNECT:
+            return EX_TEMPFAIL;
+        case NET_EIO:
+            return EX_IOERR;
+        case NET_EPROXY:
+            return EX_UNAVAILABLE;
+        case NET_ELIBFAILED:
+        default:
+            return EX_SOFTWARE;
+    }
+}
